!
!    Copyright 2007-2020 Guy Munhoven
!
!    This file is part of Medusa.
!
!    Medusa is free software: you can redistribute it and/or modify
!    it under the terms of the GNU Affero General Public License as
!    published by the Free Software Foundation, either version 3 of
!    the License, or (at your option) any later version.
!
!    Medusa is distributed in the hope that it will be useful, but
!    WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
!    See the GNU Affero General Public License for more details.
!
!    You should have received a copy of the Affero GNU General Public
!    License along with Medusa.  If not, see <https://www.gnu.org/licenses/>.
!


#ifdef CFG_MEDUSACOCOGEN
&ratelaw_config
c_name           = 'MONOD1_HYPINH1_C'
c_pp_type        = 'PP_MONOD1_HYPINH1_C'
n_param          = 6
c_expression     = '{#1} * [{#6}] * ([{#4}]/({#2} + [{#4}])) * ({#3}/({#3} + [{#5}]))'
/
! Parameter 1
&ratelaw_data
c_typecomponame  = 'RateConstant'
c_xmlattstocheck = 'type'
c_kindofparam    = 'gk'
c_dummylabel     = 'k'
/
! Parameter 2
&ratelaw_data
c_typecomponame  = 'HalfSatConstant'
c_xmlattstocheck = 'type'
c_kindofparam    = 'gk'
c_dummylabel     = 'khs'
/
! Parameter 3
&ratelaw_data
c_typecomponame  = 'InhibConstant'
c_xmlattstocheck = 'type'
c_kindofparam    = 'gk'
c_dummylabel     = 'kih'
/
! Parameter 4
&ratelaw_data
c_typecomponame  = 'ioMonod'
c_xmltagname     = 'MonodConc'
c_kindofparam    = 'io'
/
! Parameter 5
&ratelaw_data
c_typecomponame  = 'ioInhibition'
c_xmltagname     = 'InhibitionConc'
c_kindofparam    = 'io'
/
! Parameter 6
&ratelaw_data
c_typecomponame  = 'ioProportional'
c_xmltagname     = 'Proportional'
c_kindofparam    = 'io'
/
#endif
!---+----1----+----2----+----3----+----4----+----5----+----6----+----7-!
!=======================================================================
      MODULE MODLIB_MONOD1_HYPINH1_C
!=======================================================================


      ! For laws of the form (Monod + hyperbolic inhibition)
      ! k * c3 * (c1/(c_hsat + c1)) * (c_inh/(c_inh + c2))


      IMPLICIT NONE

      TYPE PP_MONOD1_HYPINH1_C
        DOUBLE PRECISION :: RateConstant       ! k
        DOUBLE PRECISION :: HalfSatConstant    ! c_hsat
        DOUBLE PRECISION :: InhibConstant      ! c_inh
        INTEGER          :: ioMonod            ! io of c1
        INTEGER          :: ioInhibition       ! io of c2
        INTEGER          :: ioProportional     ! io of c3
      END TYPE


      CONTAINS

!-----------------------------------------------------------------------
      SUBROUTINE MONOD1_HYPINH1_C(pp_param, ac, azdn, arate, darate_dac)
!-----------------------------------------------------------------------

! Monod law for constituent %ioMonod
! Proportionality for constituent %ioProportional
! Inhibition following hyperbolic law due to constituent %ioInhibition

      IMPLICIT NONE


! Argument list variables
! -----------------------

      TYPE(PP_MONOD1_HYPINH1_C),             INTENT(IN)  :: pp_param
      DOUBLE PRECISION, DIMENSION(:),        INTENT(IN)  :: ac
      DOUBLE PRECISION,                      INTENT(IN)  :: azdn
      DOUBLE PRECISION,                      INTENT(OUT) :: arate
      DOUBLE PRECISION, DIMENSION(SIZE(ac)), INTENT(OUT) :: darate_dac

      OPTIONAL :: arate, darate_dac


! Local variables
! ---------------

      INTEGER          :: io_p, io_m, io_i
      DOUBLE PRECISION :: acompo_p, acompo_m, acompo_i
      DOUBLE PRECISION :: alaw_ct, alaw_hs, alaw_ih
      DOUBLE PRECISION :: arate_p, arate_m, arate_i
      DOUBLE PRECISION :: darate_p, darate_m, darate_i


! Instructions
! ------------

      io_p = pp_param%ioProportional
      io_m = pp_param%ioMonod
      io_i = pp_param%ioInhibition

      acompo_p  = ac(io_p)
      acompo_m  = ac(io_m)
      acompo_i  = ac(io_i)
      alaw_ct  = pp_param%RateConstant
      alaw_hs  = pp_param%HalfSatConstant
      alaw_ih  = pp_param%InhibConstant


      IF (acompo_p > 0.0D+00) THEN
        arate_p = acompo_p
      ELSE
        arate_p = 0.0D+00
      ENDIF

      IF (acompo_m > 0.0D+00) THEN
        arate_m = acompo_m/(acompo_m + alaw_hs)
      ELSE
        arate_m = 0.0D+00
      ENDIF

      IF (acompo_i > 0.0D+00) THEN
        arate_i = alaw_ih/(alaw_ih + acompo_i)
      ELSE
        arate_i = 0.0D+00
      ENDIF




      IF (PRESENT(arate)) THEN

        arate = alaw_ct * arate_p * arate_m * arate_i

      ENDIF



      IF (PRESENT(darate_dac)) THEN

        IF (acompo_p > 0.0D+00) THEN
          darate_p = 1.0D+00
        ELSEIF (acompo_p == 0.0D+00) THEN
          darate_p = 1.0D+00/2.0D+00
        ELSE
          darate_p = 0.0D+00
        ENDIF

        IF (acompo_m > 0.0D+00) THEN
          darate_m = alaw_hs/((acompo_m + alaw_hs)**2)
        ELSEIF (acompo_m == 0.0D+00) THEN
          darate_m = 1.0D+00/(alaw_hs+alaw_hs)
        ELSE
          darate_m = 0.0D+00
        ENDIF

        IF (acompo_i > 0.0D+00) THEN
          darate_i = -alaw_ih/((alaw_ih + acompo_i)**2)
        ELSEIF (acompo_i == 0.0D+00) THEN
          darate_i = -1.0D+00/(2.0D+00*alaw_ih)
        ELSE
          darate_i = 0.0D+00
        ENDIF

        darate_dac(:) = 0.0D+00

        darate_dac(io_p) = alaw_ct * darate_p *  arate_m *  arate_i
        darate_dac(io_m) = alaw_ct *  arate_p * darate_m *  arate_i
        darate_dac(io_i) = alaw_ct *  arate_p *  arate_m * darate_i

      ENDIF


      RETURN

!-----------------------------------------------------------------------
      END SUBROUTINE MONOD1_HYPINH1_C
!-----------------------------------------------------------------------


!=======================================================================
      END MODULE MODLIB_MONOD1_HYPINH1_C
!=======================================================================

